#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_file.include module

This module is used for Pyramid integration
"""

__docformat__ = 'restructuredtext'

import logging

try:
    from pillow_heif import register_avif_opener, register_heif_opener
except ImportError:
    logging.getLogger('PyAMS (file)').warning("Missing pillow_heif plug-in. HEIF/HEIC image formats "
                                              "will not be supported...")
    try:
        import pillow_avif
    except ImportError:
        logging.getLogger('PyAMS (file)').warning("Missing pillow_avif plug-in. "
                                                  "AVIF image format will not be supported...")
else:
    register_avif_opener()
    register_heif_opener()


def include_package(config):
    """Pyramid package include"""

    # add translations
    config.add_translation_dirs('pyams_file:locales')

    config.scan()



#
# Copyright (c) 2008-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_file.generations.evolve2 module

This module is doing a database scan of all registered blobs to add a reference to them
into blobs manager.
"""

import logging

import transaction

from pyams_file.interfaces import IBlobReferenceManager
from pyams_site.interfaces import ISiteRoot
from pyams_utils.registry import get_local_registry, get_utility, set_local_registry
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'


LOGGER = logging.getLogger('PyAMS (file)')


def evolve(site):
    """Evolve 2: create reference for all files blobs"""
    registry = get_local_registry()
    try:
        nb_files = 0
        set_local_registry(site.getSiteManager())
        LOGGER.warning("Scanning files references for orphaned blobs...")
        references = get_utility(IBlobReferenceManager)
        for oid, refs in list(references.refs.items()):
            for ref in refs.copy():
                root = get_parent(ref, ISiteRoot)
                if root is None:
                    nb_files += 1
                    ref.remove_blob_reference()
                LOGGER.debug(f">>> removed blob reference for file {ref!r}")
            transaction.savepoint()
        LOGGER.warning(f"{nb_files} files updated")
        LOGGER.warning("Blobs references cleanup is finished. Launch *zeopack* (for ZEO storage) "
                       "or *zodbpack* (for Relstorage) command to remove all unused blobs.")
    finally:
        set_local_registry(registry)



#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_file.skin main module

This module provides functions used to render images.
"""

from datetime import datetime, timezone

from pyquery import PyQuery
from pyramid.renderers import render
from zope.dublincore.interfaces import IZopeDublinCore

from pyams_file.interfaces import ISVGImageFile
from pyams_file.interfaces.thumbnail import IThumbnails
from pyams_utils.request import check_request
from pyams_utils.url import absolute_url


__docformat__ = 'restructuredtext'


def render_svg(image, width=None, height=None, request=None, css_class='', img_class='', alt=''):
    """Render SVG file"""
    # pylint: disable=too-many-arguments
    options = {}
    if width or height:
        options['style'] = 'width: {0}{1}; height: {2}{3};'.format(
            width, 'px' if isinstance(width, int) else '',
            height, 'px' if isinstance(height, int) else '')
    else:
        options['style'] = ''
    options['css_class'] = css_class
    if alt or img_class:
        svg = PyQuery(image.data)
        if alt:
            group = PyQuery('<g></g>')
            group.append(PyQuery('<title />').text(alt))
            for child in svg.children():
                group.append(child)
            svg.empty().append(group)
        if img_class:
            svg.attr('class', img_class)
        options['svg'] = svg.outer_html()
    else:
        options['svg'] = image.data
    return render('templates/svg-render.pt', options, request)


def render_img(image, width=None, height=None, request=None, css_class='', img_class='',
               timestamp=False, alt=''):
    # pylint: disable=too-many-arguments,assignment-from-no-return
    """Render image thumbnail"""
    thumbnail = None
    thumbnails = IThumbnails(image, None)
    if thumbnails is not None:
        if width and height:
            thumbnail = thumbnails.get_thumbnail('{0}x{1}'.format(width, height))
        elif width and (width != 'auto'):
            thumbnail = thumbnails.get_thumbnail('w{0}'.format(width))
        elif height and (height != 'auto'):
            thumbnail = thumbnails.get_thumbnail('h{0}'.format(height))
    if thumbnail is None:
        thumbnail = image
    if request is None:
        request = check_request()
    url = absolute_url(thumbnail, request)
    if timestamp:
        zdc = IZopeDublinCore(thumbnail, None)
        if zdc is None:
            timestamp = datetime.now(timezone.utc).timestamp()
        else:
            timestamp = zdc.modified.timestamp()    # pylint: disable=no-member
        url += '?_={0}'.format(timestamp)
    result = '<img src="{0}" class="{1}" alt="{2}" />'.format(url, img_class, alt)
    if css_class:
        result = '<div class="{0}">{1}</div>'.format(css_class, result)
    return result


def render_image(image, width=None, height=None, request=None, css_class='', img_class='',
                 timestamp=False, alt=''):
    # pylint: disable=too-many-arguments
    """Render image"""
    if ISVGImageFile.providedBy(image):
        return render_svg(image, width, height, request, css_class, img_class, alt)
    return render_img(image, width, height, request, css_class, img_class, timestamp, alt)
